import os
import sys
import tensorflow as tf
from PIL import Image
from object_detection.utils import dataset_util

flags = tf.app.flags
flags.DEFINE_string('output_path', 'extra.tfrecord', 'Path to output TFRecord')
flags.DEFINE_string('input_name', 'extra', 'Path to output TFRecord')
flags.DEFINE_string('count', '10000', 'Path to output TFRecord')
FLAGS = flags.FLAGS

def clamp(input, min, max):
	return min if input < min else (max if input > max else input)

def printProgressBar(iteration, total, prefix = '', suffix = '', decimals = 7, fill = '█'):
	if(os.popen('stty size', 'r').read()):
		columns = int(os.popen('stty size', 'r').read().split()[1])
		length = columns - 9 - decimals
		percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
		filledLength = int(length * iteration // total)
		bar = fill * filledLength + '-' * (length - filledLength)
		print('\r%s |%s| %s%% %s' % (prefix, bar, percent, suffix), end = '\r')

def create_tfrecord(data):
	# data: [["abc.jpg", xmin, ymin, xmax, ymax], ["abc.jpg", 1, 2, 3, 4], ...]
	filename = data[0][0]
	im=Image.open(filename)
	img_width = im.size[0]
	img_height = im.size[1]

	filename = str.encode(filename)

	with open(filename, 'rb') as myfile:
		encoded_image_data = myfile.read()

	image_format = b'png'

	xmins = [clamp(int(float(x[1])) / img_width, 0, 1) for x in data]
	xmaxs = [clamp(int(float(x[3])) / img_width, 0, 1) for x in data]
	ymins = [clamp(int(float(x[2])) / img_height, 0, 1) for x in data]
	ymaxs = [clamp(int(float(x[4])) / img_height, 0, 1) for x in data]

	classes_text = [b'Digit'] * len(data)
	classes = [1] * len(data)

	tfrecord = tf.train.Example(features=tf.train.Features(feature={
		'image/height': dataset_util.int64_feature(img_height),
		'image/width': dataset_util.int64_feature(img_width),
		'image/filename': dataset_util.bytes_feature(filename),
		'image/source_id': dataset_util.bytes_feature(filename),
		'image/encoded': dataset_util.bytes_feature(encoded_image_data),
		'image/format': dataset_util.bytes_feature(image_format),
		'image/object/bbox/xmin': dataset_util.float_list_feature(xmins),
		'image/object/bbox/xmax': dataset_util.float_list_feature(xmaxs),
		'image/object/bbox/ymin': dataset_util.float_list_feature(ymins),
		'image/object/bbox/ymax': dataset_util.float_list_feature(ymaxs),
		'image/object/class/text': dataset_util.bytes_list_feature(classes_text),
		'image/object/class/label': dataset_util.int64_list_feature(classes),
	}))
	return tfrecord


def main(_):
	writer = tf.python_io.TFRecordWriter(FLAGS.output_path)

	lines = len(list(open(FLAGS.input_name + '.csv')))
	counted = 0

	with open(FLAGS.input_name + ".csv") as fp:
		fp.readline() # Get rid of labels
		line = fp.readline().strip()
		counted += 1
		while line and counted < int(FLAGS.count):
			data = line.split(",")
			record = [data]
			nextline = fp.readline().strip()
			nextdata = nextline.split(",")
			counted += 1
			printProgressBar(counted, lines)
			while data[0] == nextdata[0]:
				record.append(nextdata)
				nextline = fp.readline().strip()
				nextdata = nextline.split(",")
				counted += 1
				printProgressBar(counted, lines)
			tfrecord = create_tfrecord(record)
			create_tfrecord(record)
			writer.write(tfrecord.SerializeToString())
			line = nextline
		writer.close()

	printProgressBar(lines, lines)
	print("Done.")

if __name__ == '__main__':
	tf.app.run()
